!!****m* HAPLOTYPE/m_qtlmap_haplotype_ldla
!!  NAME
!!    m_qtlmap_haplotype_ldla
!!  DESCRIPTION
!!
!!  NOTES
!!
!!  BUGS
!!
!!  HISTORY
!!
!!  SEE ALSO
!!
!!  COPYRIGHT
!!***
!! You can use this space for remarks that should not be included
!! in the documentation.
!!/
module m_qtlmap_haplotype_ldla
    use m_qtlmap_tools
    use m_qtlmap_haplotype_data
    use m_qtlmap_data
    use m_qtlmap_log
    use m_qtlmap_output_handler

    implicit none
    save
    private 

!!****v* m_qtlmap_haplotype_ldla/haplo
!! NAME
!!   haplo
!! DESCRIPTION
!!   list of the possible haplotype (with unknwon allele) at the current position
!! DIMENSIONS
!!   1: index of the haplotype
!!   2: index of the allele (1 < index allele < longap)
!! NOTES
!! SOURCE
    integer(kind=KIND_PHENO), dimension (:,:)    , allocatable   :: haplo
!!***

!!****v* m_qtlmap_haplotype_ldla/race_h
!! NAME
!!   race_h
!! DESCRIPTION
!!   breed origin of haplotype haplo
!! DIMENSIONS
!!   1: index of the haplotype
!! NOTES
!! SOURCE
     character(len=LEN_DEF), dimension (:)    , allocatable      :: race_h
!!***

!!****v* m_qtlmap_haplotype_ldla/haplo_complet
!! NAME
!!   haplo_complet
!! DESCRIPTION
!!   list of the possible haplotype (without unknwon allele) at the current position
!! DIMENSIONS
!!   1: index of the complete haplotype
!!   2: index of the allele (1 < index allele < longhap)
!! NOTES
!!  see liste_haplo_complet
!! SOURCE
    integer(kind=KIND_PHENO), dimension (:,:)    , allocatable   :: haplo_complet
!!***

!!****v* m_qtlmap_haplotype_ldla/race_haplo_complet
!! NAME
!!   race_haplo_complet
!! DESCRIPTION
!!   breed origin of haplotype haplo_complet
!! DIMENSIONS
!!   1: index of the complete haplotype
!! NOTES
!! SOURCE
     character(len=LEN_DEF), dimension (:)    , allocatable      :: race_haplo_complet
!!***

!!****v* m_qtlmap_haplotype_ldla/gamete
!! NAME
!!   gamete
!! DESCRIPTION
!!
!! DIMENSIONS
!!   1: 1,2
!!   2: marker (1 <= <= longhap)
!!   3: 2**longhap
!! NOTES
!!  gamete(1,lk,j_gam) l allele paternel au lk eme marqueur pour le j_gam eme gamete
!!	gamete(2,lk,j_gam) l allele maternel au lk eme marqueur pour le j_gam eme gamete
!! SOURCE
    integer(kind=KIND_PHENO), dimension (:,:,:) ,allocatable :: gamete
!!***

!!****v* m_qtlmap_haplotype_ldla/loc_haplo
!! NAME
!!   loc_haplo
!! DESCRIPTION
!!
!!                !-----!
!!  A A A T T T A A A A T T T T A T A T A T A T T T T
!!                | | | |
!!                | | | ---------------
!!                | | -------------   |
!!                | -------       |   |
!!         loc_haplo(1)  |       |   |
!!                 loc_haplo(2)  |   |
!!                      loc_haplo(3) |
!!                           loc_haplo(4)
!!
!! DIMENSIONS
!!   1: index of the marker (size:longhap)
!! NOTES
!!  loc_haplo, le vecteur de dimension longhap qui contient les numeros des marqueurs
!!  pour l haplotype entourant la position testee dx, et lk le premier marqueur flanquant a droite
!! SOURCE
    integer                 , dimension (:)      , allocatable   :: loc_haplo
!!***

!!****v* m_qtlmap_haplotype_ldla/count_haplo_complet
!! NAME
!!   count_haplo_complet
!! DESCRIPTION
!!
!! DIMENSIONS
!!   1:
!! NOTES
!! SOURCE
    real(kind=dp)        , dimension (:)      , allocatable   :: count_haplo_complet
!!***

!!****v* m_qtlmap_haplotype_ldla/pb_haplo_complet
!! NAME
!!   pb_haplo_complet
!! DESCRIPTION
!!
!! DIMENSIONS
!!   1:
!!   2:
!!   3:
!! NOTES
!! SOURCE
    real(kind=dp)        , dimension (:)      , allocatable   :: pb_haplo_complet
!!***

!!****v* m_qtlmap_haplotype_ldla/tab_haplo_complet
!! NAME
!!   tab_haplo_complet
!! DESCRIPTION
!!
!! DIMENSIONS
!!   1:
!!   2:
!! NOTES
!! SOURCE
    integer(kind=KIND_PHENO), dimension (:,:)    , allocatable   :: tab_haplo_complet
!!***

!!****v* m_qtlmap_haplotype_ldla/nb_haplo_possible
!! NAME
!!   nb_haplo_possible
!! DESCRIPTION
!!
!! DIMENSIONS
!!   1:
!! NOTES
!! SOURCE
    integer              , dimension (:)    , allocatable     :: nb_haplo_possible
!!***
!!****v* m_qtlmap_haplotype_ldla/comp_haplo
!! NAME
!!   comp_haplo
!! DESCRIPTION
!!
!! DIMENSIONS
!!   1:
!!   2:
!! NOTES
!! SOURCE
    logical                 , dimension (:,:)    , allocatable   :: comp_haplo
!!***

    !$omp threadprivate (haplo,haplo_complet,race_h,race_haplo_complet,gamete,loc_haplo,count_haplo_complet)
    !$omp threadprivate (pb_haplo_complet,tab_haplo_complet,comp_haplo,nb_haplo_possible)

    public :: set_haplo_for_ldla


    public  :: set_allocation_ldla
    public  :: free_internal_struct_ldla
    public  :: liste_haplo
    public  :: liste_haplo_complet
    public  :: proba_haplo_complet
    public  :: tri_haplo_complet
    public  :: set_haplo_final
    public  :: sort_haplo
    private :: local_gamete
    private :: point_gamete


    public :: liste_gamete
    public :: set_tab_ibs
    public :: liste_shared_haplo
    public :: bilan_shared_haplo

   contains

!!****f* m_qtlmap_haplotype_ldla/set_allocation
!! NAME
!!    set_allocation
!! DESCRIPTION
!!
!! NOTES
!!
!! SOURCE
      subroutine set_allocation_ldla
        integer  :: stat,value_alloc_1,value_alloc_2,c

      
      allocate (loc_haplo(longhap),STAT=stat)
      call check_allocate(stat,'loc_haplo [m_qtlmap_haplotype_ldla]')
      allocate (nb_gam(nd),STAT=stat)
      call check_allocate(stat,'nb_gam [m_qtlmap_haplotype_ldla]')
      allocate (nb_gam_pere(np,2),STAT=stat)
      call check_allocate(stat,'nb_gam_pere [m_qtlmap_haplotype_ldla]')
      allocate (nb_gam_mere(maxval(ngenom(:,nm+1)),2),STAT=stat)
      call check_allocate(stat,'nb_gam_mere [m_qtlmap_haplotype_ldla]')
      allocate (tab_shared_haplo(np,2,1+maxval(nmk)),STAT=stat)
      call check_allocate(stat,'tab_shared_haplo [m_qtlmap_haplotype_ldla]')

      value_alloc_1= (2*(np+nm)+nd*2**longhap)*NB_RACES
      value_alloc_2=maxval(nall)**(2*longhap)+1

      allocate (haplo(value_alloc_1,longhap),STAT=stat)
      call check_allocate(stat,'haplo [m_qtlmap_haplotype_ldla]')
!      allocate (haplo_complet(value_alloc_1,value_alloc_2),STAT=stat)
!      call check_allocate(stat,'haplo_complet [m_qtlmap_haplotype_ldla]')
      allocate (nb_haplo_possible(value_alloc_1),STAT=stat)
      call check_allocate(stat,'nb_haplo_possible [m_qtlmap_haplotype_ldla]')  
      !add race info
      allocate (race_h(value_alloc_1),STAT=stat)
      call check_allocate(stat,'race_h [m_qtlmap_haplotype_ldla]')
      allocate (race_haplo_complet(value_alloc_1),STAT=stat)
      call check_allocate(stat,'race_haplo_complet [m_qtlmap_haplotype_ldla]')
      allocate (count_haplo_complet(value_alloc_1),STAT=stat)
      call check_allocate(stat,'count_haplo_complet [m_qtlmap_haplotype_ldla]')    
      allocate (prob_gam(nd,value_alloc_2),STAT=stat)
      call check_allocate(stat,'prob_gam [m_qtlmap_haplotype_ldla]')
      allocate (nb_gam_desc(nd),STAT=stat)
      call check_allocate(stat,'nb_gam_desc [m_qtlmap_haplotype_ldla]')
      allocate (gamete(2,longhap,value_alloc_2),STAT=stat)
      call check_allocate(stat,'gamete [m_qtlmap_haplotype_ldla]')
      allocate (tab_IBS(nd,maxval(nmk),2),STAT=stat)
      call check_allocate(stat,'tab_IBS [m_qtlmap_haplotype_ldla]')
      allocate (name_haplo(value_alloc_2),STAT=stat)
      call check_allocate(stat,'name_haplo [m_qtlmap_haplotype_ldla]')
      allocate (name_haplo_reduit(value_alloc_2),STAT=stat)
      call check_allocate(stat,'name_haplo_reduit [m_qtlmap_haplotype_ldla]')
      allocate (race_haplo_reduit(value_alloc_2),STAT=stat)
      call check_allocate(stat,'race_haplo_reduit [m_qtlmap_haplotype_ldla]')
      allocate (name_haplo_complet(value_alloc_2),STAT=stat)
      call check_allocate(stat,'name_haplo_complet [m_qtlmap_haplotype_ldla]')
      allocate (count_haplo(value_alloc_2),STAT=stat)
      call check_allocate(stat,'count_haplo [m_qtlmap_haplotype_ldla]')
      allocate (pb_haplo_complet(10*NB_RACES),STAT=stat)
      call check_allocate(stat,'pb_haplo_complet [m_qtlmap_haplotype_ldla:proba_haplo_complet]')
      allocate (comp_haplo(10,10*NB_RACES),STAT=stat)
      call check_allocate(stat,'comp_haplo [m_qtlmap_haplotype_ldla]')
      allocate (liste_haplo_reduit(10*NB_RACES))
      allocate (tab_haplo_complet(10*NB_RACES ,longhap),STAT=stat)
      call check_allocate(stat,'liste_haplo_complet :: tab_haplo_complet [m_qtlmap_haplotype_ldla:liste_haplo_complet]')

      allocate (comp_haplo_reduit(10,50),STAT=stat)
      call check_allocate(stat,'comp_haplo_reduit [m_qtlmap_haplotype_ldla]')
      allocate (pb_haplo_reduit(50),STAT=stat)
      call check_allocate(stat,'pb_haplo_reduit [m_qtlmap_haplotype_ldla]')

      allocate (num_haplo_pere(np,2,10),STAT=stat)
      call check_allocate(stat,'num_haplo_pere [m_qtlmap_haplotype_ldla]')

      allocate (num_haplo_mere(maxval(ngenom(:,nm+1)),2,nb_haplo_complet),STAT=stat)
      call check_allocate(stat,'num_haplo_mere [m_qtlmap_haplotype_ldla]')

      allocate (num_haplo_desc(nd,100),STAT=stat)
      call check_allocate(stat,'num_haplo_desc [m_qtlmap_haplotype_ldla]')

      allocate (pb_haplo_desc(nd,100),STAT=stat)
      call check_allocate(stat,'pb_haplo_desc [m_qtlmap_haplotype_ldla]')


    end subroutine set_allocation_ldla
!!***

!!****f* m_qtlmap_haplotype_ldla/free_internal_struct_ldla
!! NAME
!!    free_internal_struct_ldla
!! DESCRIPTION
!!
!! NOTES
!!
!! SOURCE
    subroutine free_internal_struct_ldla

        if (allocated(haplo))               deallocate (haplo)
        if (allocated(haplo_complet))       deallocate (haplo_complet)
        if (allocated(race_h))              deallocate (race_h)
        if (allocated(loc_haplo))           deallocate (loc_haplo)
        if (allocated(race_haplo_complet))  deallocate (race_haplo_complet)  
        if (allocated(gamete))              deallocate (gamete) 
        if (allocated(count_haplo_complet)) deallocate (count_haplo_complet)
        if (allocated(nb_gam))              deallocate (nb_gam)
        if (allocated(prob_gam))            deallocate (prob_gam)
        if (allocated(num_haplo_pere))      deallocate (num_haplo_pere)
        if (allocated(num_haplo_mere))      deallocate (num_haplo_mere)
        if (allocated(num_haplo_desc))      deallocate (num_haplo_desc)
        if (allocated(nb_gam_pere))         deallocate (nb_gam_pere)
        if (allocated(nb_gam_mere))         deallocate (nb_gam_mere)
        if (allocated(nb_gam_desc))         deallocate (nb_gam_desc)
        if (allocated(tab_IBS))             deallocate (tab_IBS)
        if (allocated(name_haplo))          deallocate (name_haplo)
        if (allocated(name_haplo_reduit))   deallocate (name_haplo_reduit)
        if (allocated(race_haplo_reduit))   deallocate (race_haplo_reduit)
        if (allocated(name_haplo_complet))  deallocate (name_haplo_complet)
        if (allocated(count_haplo))         deallocate (count_haplo)
        if (allocated(tab_shared_haplo))    deallocate( tab_shared_haplo)
        if (allocated(comp_haplo))          deallocate (comp_haplo)
        if (allocated(comp_haplo_reduit))   deallocate (comp_haplo_reduit)
        if (allocated(liste_haplo_reduit))  deallocate (liste_haplo_reduit)
        if (allocated(pb_haplo_reduit))     deallocate (pb_haplo_reduit)
        if (allocated(pb_haplo_complet))    deallocate (pb_haplo_complet)
        if (allocated(tab_haplo_complet))   deallocate (tab_haplo_complet)
        if (allocated(nb_haplo_possible))   deallocate (nb_haplo_possible)
        if (allocated(pb_haplo_desc))       deallocate (pb_haplo_desc)


    end subroutine free_internal_struct_ldla
!!***

!!****f* m_qtlmap_haplotype_ldla/set_haplo_for_ldla
!! NAME
!!    set_haplo_for_ldla
!! DESCRIPTION
!!
!! NOTES
!!  set_haplo construit la liste des haplotypes de longueur longhap
!!
!! Utilise nchr,posi,pas,nmk de m_qtlmap_data
!! utilise longhap de m_qtlmap_data
!! SOURCE
      subroutine set_haplo_for_ldla(c,dx,n,hsire,hdam)
      integer c,i,j,k
      double precision dx
      integer, intent (in) :: n
      logical,intent(in)   :: hsire,hdam

      if (.not. check_change_marker_windows(c,dx)) return
!
!  longhap est la longueur de l haplotype dont on estime l effet
!  longhap est defini en nombre de marqueurs ( multiple de 2)  
 !       PROB_HAPLO_MIN=0.0d0
        !call free_internal_struct_ldla
        !call set_allocation_ldla
        call liste_haplo(c,dx,n,hsire,hdam)
        call liste_haplo_complet(c)
        call proba_haplo_complet(c,dx)
        call tri_haplo_complet(c,dx)
        call set_haplo_final(c,n,hsire,hdam)
      end subroutine set_haplo_for_ldla
!!***

!!****f* m_qtlmap_haplotype_ldla/set_tab_ibs
!! NAME
!!    set_tab_ibs
!! DESCRIPTION
!!
!! NOTES
!! set_tab_ibs construit la liste des haplotypes correspondant au chromosome entier
!! SOURCE
      subroutine set_tab_ibs(c) 
!
! set_tab_ibs construit la liste des haplotypes correspondant au chromosome entier
!
! Utilise nchr,posi,pas,nmk de m_qtlmap_data
! utilise longhap de m_qtlmap_data
!
      integer, intent (in) :: c
!
!  longhap est la longueur de l haplotype dont on estime l effet
!  longhap est defini en nombre de marqueurs ( multiple de 2)  
        !call free_internal_struct_ldla
        call set_allocation_ldla 
        call liste_gamete(c)
       return
 
     end subroutine set_tab_ibs
!!***

!!****f* m_qtlmap_haplotype_ldla/sort_haplo
!! NAME
!!    sort_haplo
!! DESCRIPTION
!!
!! NOTES
!!  sort_haplo trie les haplotypes dans la liste haplo et les recodifie
!!  chaque reproducteur est affecte de ses haplotype recodifies
!!  la liste des noms d'haplotype (name_haplo) est cree
!!
!! Utilise np,nm,ndm,estfem,repfem de m_qtlmap_data
!! utilise ngenom de m_qtlmap_haplotpe_data
!! SOURCE
     subroutine sort_haplo(c,n,hsire,hdam)
     logical :: hsire,hdam
! dim allocatable
     integer(kind=KIND_PHENO)  , dimension (:,:)    , allocatable    :: kept_haplo
     integer                   , dimension (:)      , allocatable    :: corr_haplo
     integer                   , dimension (:)      , allocatable    :: corr_corr_haplo
     integer                   , dimension (:)      , allocatable    :: inv_corr_haplo
! divers
      integer c
      integer i,i_haplo,i_kept,ip,j,jm,j_gam,kd,geno,previous,j_haplo
      integer nb_kept,value_alloc
      integer  :: stat,lstr1,lstr2
      integer, intent (in) :: n
      character(len=LEN_DEF) :: buf
      double precision dencount
      logical inconnu, new

! allocation
      value_alloc=nb_haplo
      allocate (kept_haplo(value_alloc,longhap),STAT=stat)
      call check_allocate(stat,'kept_haplo [m_qtlmap_haplotype_LD:sort_haplo]')

      allocate (corr_haplo(value_alloc),STAT=stat)
      call check_allocate(stat,'corr_haplo [m_qtlmap_haplotype_LD:sort_haplo]')
      corr_haplo=0

      allocate (corr_corr_haplo(value_alloc),STAT=stat)
      call check_allocate(stat,'corr_corr_haplo [m_qtlmap_haplotype_LD:sort_haplo]')

      allocate (inv_corr_haplo(value_alloc),STAT=stat)
      call check_allocate(stat,'inv_corr_haplo [m_qtlmap_haplotype_LD:sort_haplo]')



! on trie les haplotypes trouves en eliminant les redondances
!
! la liste reduite sera dans kept_haplo
! de longueur nb_kept
! le i eme ancien haplotype est a la corr_haplo(i) eme place dans kept_haplo
!
      name_haplo=''
      kept_haplo(1,1)=haplo(1,1)
      lstr1=len(trim(get_pheno(c,haplo(1,1))))
      name_haplo(1)(1:lstr1)=trim(get_pheno(c,haplo(1,1)))

      do i=2,longhap
         kept_haplo(1,i)=haplo(1,i)
         lstr2=len(trim(get_pheno(c,haplo(1,i))))
         name_haplo(1)(lstr1+1:lstr1+lstr2)=trim(get_pheno(c,haplo(1,i)))
         lstr1=lstr1+lstr2
      end do !i
      corr_haplo(1)=1
      nb_kept=1

      do i_haplo=2, nb_haplo
        inconnu=.true.
        do i_kept =1,nb_kept
          new=.false.         
          do i=1,longhap
            if(kept_haplo(i_kept,i) /= haplo(i_haplo,i))new=.true.
            if(new)exit 
          end do
          if(.not.new) previous=i_kept
          if(.not.(inconnu.and.new))inconnu=.false.
        end do ! i_kept
        if(inconnu)then
          nb_kept=nb_kept+1
          kept_haplo(nb_kept,1)=haplo(i_haplo,1)
          lstr1=len(trim(get_pheno(c,haplo(i_haplo,1))))
          name_haplo(nb_kept)(1:lstr1)=trim(get_pheno(c,haplo(i_haplo,1)))
          do i=2,longhap
            kept_haplo(nb_kept,i)=haplo(i_haplo,i)
            lstr2=len(trim(get_pheno(c,haplo(i_haplo,i))))
            name_haplo(nb_kept)(lstr1+1:lstr1+lstr2)=trim(get_pheno(c,haplo(i_haplo,i)))
            lstr1=lstr1+lstr2
            buf=get_pheno(c,haplo(i_haplo,i))
          end do !i
          corr_haplo(i_haplo)=nb_kept
        else
          corr_haplo(i_haplo)=previous
        end if
      end do !i_haplo
!
!  Compteur d'haplotypes
!
      dencount=1.d0/dble(nb_haplo)
      count_haplo(:)=0.d0
      do i_haplo=1,nb_haplo
        count_haplo(corr_haplo(i_haplo))=count_haplo(corr_haplo(i_haplo))+dencount
      end do

      nb_max_haplo=nb_kept

      if(nb_max_haplo > NB_HAPLO_PRIOR)then
       call stop_application('more haplotypes found than expected ['//trim(str(nb_haplo))//&
                             '] opt_nb_haplo_prior must be increased')
      end if

!
!  regroupement des haplotypes rares dans le paquet des donnees manquantes
!
       nb_kept=1
       corr_corr_haplo(1)=1
       inv_corr_haplo(1)=1
       do i_haplo=2,nb_max_haplo
        if(count_haplo(i_haplo) < PROB_HAPLO_MIN) then
          corr_corr_haplo(i_haplo)=1
          inv_corr_haplo(1)=1!peut etre que c est i_haplo....
        else
          nb_kept=nb_kept+1
          corr_corr_haplo(i_haplo)=nb_kept
          inv_corr_haplo(nb_kept)=i_haplo
        end if
      end do ! i_haplo
      count_haplo(:)=0.d0
      do i_haplo=1,nb_haplo
        j_haplo=corr_haplo(i_haplo)
        count_haplo(corr_corr_haplo(j_haplo))=count_haplo(corr_corr_haplo(j_haplo))+dencount
      end do
      nb_max_haplo=nb_kept

!  creation d un indice d haplotype par reproducteur
!
!  pour les peres
       j_haplo=1
       if(hsire)then
       do ip=1,np
         do j=1,2
           j_haplo=j_haplo+1
           num_haplo_pere(ip,j,1)=corr_corr_haplo(corr_haplo(j_haplo))
         end do !j
       end do !ip
       end if ! opt_model

       if(hdam)then
!
!  pour les meres 
!
       do jm=1,nm
         if(estfem(repfem(jm)))then
!
!  meres phasees
!
           do geno=ngenom(c,jm)+1,ngenom(c,jm+1)
             do j=1,2
               j_haplo=j_haplo+1
               num_haplo_mere(geno,j,1)=corr_corr_haplo(corr_haplo(j_haplo))
             end do !j
           end do !geno
         else
! 
!   non phasees
           do kd=ndm(jm)+1,ndm(jm+1)
             if(presentg(c,kd))then
             do j_gam=1,nb_gam(kd)
               j_haplo=j_haplo+1
!               num_haplo_desc(kd,j_gam,1)=corr_corr_haplo(corr_haplo(j_haplo))
               num_haplo_desc(kd,j_gam)=corr_corr_haplo(corr_haplo(j_haplo))
             end do !j_gam
             end if
           end do !kd
         end if
       end do !jm
       end if ! opt_model

      deallocate(kept_haplo)
      deallocate(corr_haplo)
      deallocate(corr_corr_haplo)
      deallocate(inv_corr_haplo)

     end  subroutine sort_haplo
!!***

!!****f* m_qtlmap_haplotype_ldla/check_change_maker_windows
!! NAME
!!    liste_haplo
!! DESCRIPTION
!!
!! NOTES
!!
!! SOURCE
     function check_change_marker_windows(c,dx) result(change)
       integer       , intent(in) :: c
       real(kind=dp) , intent(in) :: dx

       logical :: change
       integer :: lkmin,lkmax,lk

       change = .true.
       if (.not. allocated(loc_haplo) ) then
      !   print *,'not allocated...'
         return
       end if
      ! lkmax est la limite  pour le marqueur flanquant l haplotype sur sa droite
!
       lkmin=longhap/2
       if ( lkmin == 0 ) lkmin = 1
       lkmax=nmk(c)-lkmin+1
!  la position testee doit etre compatible avec la longueur de l'haplotype
!

       if((dx < posi(c,lkmin)) .or. (dx > posi(c,lkmax))) then
         call stop_application("check_change_marker_windows : incompatibilty between haplotype length and tested position.")
       end if

       lk=lkmin +1

       do while (dx > posi(c,lk) .and. lk < lkmax)
         lk=lk+1
       end do

       if ( loc_haplo(1) == lk-lkmin ) change=.false.

     end function check_change_marker_windows
!!***


!!****f* m_qtlmap_haplotype_ldla/liste_haplo
!! NAME
!!    liste_haplo
!! DESCRIPTION
!!
!! NOTES
!!  liste_haplo cherche  les haplotypes de longeur longhap possibles
!!  autour de la position dx du chromosome c
!! Utilise nmk,np,posi,correp,nm,repfem,correm,estfem    de m_qtlmap_data
!! Utilise genotyp,ngenom,genotypm                       de m_qtlmap_haplotype_data
!! SOURCE
    subroutine liste_haplo(c,dx,n,hsire,hdam)
      integer       , intent(in) :: c
      real(kind=dp) , intent(in) :: dx
      integer, intent (in)       :: n
      logical       , intent(in) :: hsire,hdam

      integer lkmax,lkmin,i,ip,j,jm,geno,j_gam,kd,lk

      nb_gam=0

! lkmax est la limite  pour le marqueur flanquant l haplotype sur sa droite
!
       lkmin=longhap/2
       if ( lkmin == 0 ) lkmin = 1
       lkmax=nmk(c)-lkmin+1
!  la position testee doit etre compatible avec la longueur de l'haplotype
!

       if((dx < posi(c,lkmin)) .or. (dx > posi(c,lkmax))) then
         call stop_application("liste_haplo : incompatibilty between haplotype length and tested position.")
       end if

!
! initialisation de loc_haplo, le vecteur de dimension longhap qui contient les numeros des marqueurs
! pour l haplotype entourant la position testee dx, et lk le premier marqueur flanquant a droite
       do i=1,longhap
         loc_haplo(i)=i
       end do !i

       lk=loc_haplo(lkmin) +1

       do while (dx > posi(c,lk) .and. lk < lkmax)
         lk=lk+1
       end do

!
! lk est le premier marqueur a droite de dx
       do i=1,longhap
         loc_haplo(i)=lk-lkmin-1+i
       end do  !i 

       !print *,loc_haplo
!
!  le premier haplotype dans la liste sera celui des donnees manqantes, il y en aura un par race quand il y a plusieurs races
!
       nb_haplo=0
       race_h='UNKNOWN'
       do j=1,NB_RACES
          nb_haplo=nb_haplo+1
          do i=1,longhap
           haplo(nb_haplo,i)=nmanque
         end do !i
           race_h(nb_haplo)=nom_race(j)
       enddo
!
!  haplotypes trouves chez les peres
!
       if(hsire)then
       do ip=1,np
         do j=1,2
           nb_haplo=nb_haplo+1
           do i=1,longhap
             haplo(nb_haplo,i)=genotyp(c,loc_haplo(i),correp(ip),j)             
           end do !i
!! Si les parents du père sont connus (reppere.ne.INT_NOT_DEFINED) et sont 
!! soit de meme race soit génotypés sur au moins un marqueur
           if (raceFileDefined) then
               if (reppere(ip).eq.INT_NOT_DEFINED) then 
                    call stop_application('Both parents of sires and dams have to be defined in the ped file '//&
           'to use breed origin of haplotypes. The parents of sire  '//trim(pere(ip))//&
                    ' are not in the ped file! ')
               else          
                 if (( racep(reppere(ip))==racem(reppere(ip)) )  &
                .or. (phasp(c,ip)                             )) then
                    if (j==1) race_h(nb_haplo)= racep(reppere(ip))
                    if (j==2) race_h(nb_haplo)= racem(reppere(ip))
                 endif
              endif 
            endif
         end do !j

       end do !ip
       end if ! hsire

       if(hdam)then

!
!  haplotypes trouves chez les meres phasees
!
  
       do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
         if(estfem(repfem(jm)))then
!
!  cas des femelles dont on estime la phase
!
           do geno=ngenom(c,jm)+1,ngenom(c,jm+1)
             do j=1,2
               nb_haplo=nb_haplo+1
               do i=1,longhap
                  haplo(nb_haplo,i)=genotypm(c,loc_haplo(i),geno,j)
               end do !i
!! Si les parents de la mère sont connus (repmere.ne.INT_NOT_DEFINED) et sont 
!! soit de meme race soitgénotypés sur au moins un marqueur
           if (raceFileDefined) then
               if (repmere(jm).eq.INT_NOT_DEFINED) then 
                   call stop_application ("Both parents of sires and dams have to be defined in the ped file "//&
           "to use breed origin of haplotypes. the parents of dam  "//trim(mere(jm))//&
                    " are not in the ped file!")
               else          
                  if ( ( racep(repmere(jm))==racem(repmere(jm)) ) &
                  .or. (phasm(c,jm)                                                               )) then
                    if (j==1)  race_h(nb_haplo)= racep(repmere(jm))
                    if (j==2)  race_h(nb_haplo)= racem(repmere(jm))
                end if
               end if
             end if
             end do !j
           end do !geno
! 
!  cas des femelles non phasees (issues d'une seule race lorsqu'elle est renseignée)
!
         else 
            geno=0
            do kd=ndm(jm)+1,ndm(jm+1)
              if(presentg(c,kd)) then
                call local_gamete(ip,jm,geno,kd,c,dx)
                 do j_gam=1,nb_gam(kd)
                    nb_haplo=nb_haplo+1
                    do i=1,longhap
                      haplo(nb_haplo,i)=gamete(2,i,j_gam)
                    end do !i
                    if (raceFileDefined) then
                       if (repmere(jm).eq.INT_NOT_DEFINED) then 
                          call stop_application ("Both parents of sires and dams have to be defined in the ped file "//&
                          "to use breed origin of haplotypes. the parents of dam  "//trim(mere(jm))//&
                          " are not in the ped file!")
                          STOP
                       else          
                        if (racep(repmere(jm))==racem(repmere(jm))) race_h(nb_haplo)= racep(repmere(jm))
                       endif
                    endif
                end do !j_gam
              end if !presentg
            end do !kd
         end if

         end do !jm
        end do !ip
         end if ! hdam
!ATTENTION: lorsqu'un haplotype n'a pas une race connue -->le programme ne continue pas
! normalement cette erreur a été detectée avant
        
         do j=1,nb_haplo
            if (raceFileDefined.and.race_h(j)=='UNKNOWN') then
              call stop_application("An haplotype has an unknown breed origin, PLEASE check if you give"//&
              " the breeds of all funder parents!")
            endif
            !print *, 'HAPLO_pop:',j, race_h(j), (haplo(j,i),i=1,longhap)
         enddo
         end subroutine liste_haplo
!!***

!!****f* m_qtlmap_haplotype_ldla/liste_haplo_complet
!! NAME
!!    liste_haplo_complet
!! DESCRIPTION
!!
!! NOTES
!! liste_haplo_complet etablit la liste des haplotypes complets possible
!!
!!  en entree, il lui faut
!!	loc_haplo , longhap  et loc_haplo  créés par la routine liste_haplo du module m_qtlmap_haplotype_ldla
!!       nall et alleles créés par la routine set_allele_info_vector du module m_qtlmap_genotype
!!  en sortie il fournit
!!	tab_haplo_complet et nb_haplo_complet
!! SOURCE
     subroutine liste_haplo_complet(c)
      integer       , intent(in) :: c
      integer i1,i2,j,j1,j2
      integer nb_row,nb_col
      integer  :: stat

! allocation

      nb_haplo_complet = 1
      do j=1,longhap
        nb_haplo_complet = nb_haplo_complet * nall(c,loc_haplo(j))
      end do !j


      if ( (size(tab_haplo_complet,1)<nb_haplo_complet*NB_RACES) ) then
        deallocate (tab_haplo_complet)
        allocate (tab_haplo_complet(nb_haplo_complet*NB_RACES ,longhap),STAT=stat)
        call check_allocate(stat,'liste_haplo_complet :: tab_haplo_complet [m_qtlmap_haplotype_ldla:liste_haplo_complet]')
      end if



      tab_haplo_complet = nmanque


      do i1 = 1,nall(c,loc_haplo(1))
        tab_haplo_complet(i1,1) = set_pheno(c,alleles(c,loc_haplo(1),i1))
      end do !i1


      nb_row=nall(c,loc_haplo(1))
      nb_col=1
 
      do j=2,longhap

        do i1 = 1,nb_row
          tab_haplo_complet(i1,nb_col+1) = set_pheno(c,alleles(c,loc_haplo(j),1))
        end do !i1

        do j1= 2,nall(c,loc_haplo(j))

          do i2 = 1,nb_row
            do j2= 1,nb_col
              tab_haplo_complet(i2+(j1-1)*nb_row,j2) = tab_haplo_complet(i2,j2)
            end do !j2
              tab_haplo_complet(i2+(j1-1)*nb_row,nb_col+1) = set_pheno(c,alleles(c,loc_haplo(j),j1))
          end do !i2

        end do !j1
          nb_row=nb_row * nall(c,loc_haplo(j))
          nb_col= nb_col+1
      end do !j

 
      
      end subroutine liste_haplo_complet
!!***

!!****f* m_qtlmap_haplotype_ldla/proba_haplo_complet
!! NAME
!!    proba_haplo_complet
!! DESCRIPTION
!!
!! NOTES
!!  proba_haplo_complet etablit les probabilites des haplotypes complets
!!
!! en entree, il lui faut
!!	nb_haplo_complet et tab_haplo_complet créés par la routine liste_haplo_complet du module m_qtlmap_haplotype_ldla
!!       haplo et nb_haplo créés par la routine set_allele_info_vector du module m_qtlmap_genotype
!! en sortie il fournit pb_haplo_complet
!!
!! HISTORY
!!   correction BUG : leak memory 05/11/2010
!! SOURCE
     subroutine proba_haplo_complet(c,dx)
      integer       , intent(in) :: c
      real(kind=dp) , intent(in) :: dx
      double precision erreur
      integer iht,jhc,i,iter,ri
      logical compatible
      integer  :: stat
      real(kind=dp)  :: S
! dim allocatable
 
      real(kind=dp)     , dimension (:)      , allocatable    :: phc
      real(kind=dp)     , dimension (:)      , allocatable    :: pht

! local allocation

      allocate (phc(nb_haplo_complet*NB_RACES),STAT=stat)
      call check_allocate(stat,'phc [m_qtlmap_haplotype_ldla:proba_haplo_complet]')
      allocate (pht(nb_haplo),STAT=stat)
      call check_allocate(stat,'pht [m_qtlmap_haplotype_ldla:proba_haplo_complet]')

      !resize if too small
      if ( size(comp_haplo,1) < nb_haplo .or. size(comp_haplo,2) < nb_haplo_complet*NB_RACES ) then
         deallocate (comp_haplo)
         allocate (comp_haplo(nb_haplo,nb_haplo_complet*NB_RACES))
      end if

     if ( size(pb_haplo_complet) < nb_haplo_complet*NB_RACES ) then
         deallocate (pb_haplo_complet)
         allocate (pb_haplo_complet(nb_haplo_complet*NB_RACES))
     end if

 
      pb_haplo_complet = 0.d0
      comp_haplo=.false.
      race_haplo_complet='UNKNOWN'
      do ri=1,NB_RACES
         do jhc = 1,nb_haplo_complet
            do i=1,longhap
              tab_haplo_complet(jhc+nb_haplo_complet*(ri-1),i)=tab_haplo_complet(jhc,i)
              race_haplo_complet(jhc+nb_haplo_complet*(ri-1))=nom_race(ri)
            enddo
          enddo
       enddo
       nb_haplo_complet= nb_haplo_complet*NB_RACES

!
!  creation de la table de compatibilites entre les haplotypes observes et les haplotypes complets possibles
!
      do iht=1,nb_haplo
        do jhc = 1,nb_haplo_complet
          compatible =.true.
         !haplotypes de races différentes
          if (race_haplo_complet(jhc) /= race_h(iht)) then
              compatible = .false.
              !exit
          else
         !haplotypes différents de meme race
            do i=1,longhap
              if( (haplo(iht,i) /= tab_haplo_complet(jhc,i)) .and. (haplo(iht,i) /= nmanque) )then
                compatible = .false.
                !exit
              end if
            end do !i
          endif
          if(compatible) comp_haplo(iht,jhc)=.true.
        end do !jhc
      end do !iht


!
!  Algorithme EM pour l'estimation des probabilites des haplotypes complets
!
      pb_haplo_complet =1.d0 / dble(nb_haplo_complet)

      erreur=1.d0
      iter=0

      do while( erreur > EPS_EM)

        pht=0.d0
        do iht = 1, nb_haplo
          do jhc=1,nb_haplo_complet
           if(comp_haplo(iht,jhc)) pht(iht)=pht(iht)+pb_haplo_complet(jhc)
          end do !jhc
        end do !iht
 

        erreur=0.d0
        phc=0.d0 
        do jhc=1,nb_haplo_complet        
          do iht = 1, nb_haplo
            if(comp_haplo(iht,jhc)) phc(jhc)=phc(jhc)+pb_haplo_complet(jhc)/pht(iht)
          end do !iht
          phc(jhc)=phc(jhc)/dble(nb_haplo)
          erreur=erreur+dabs(phc(jhc)-pb_haplo_complet(jhc))
        end do !jhc

        iter=iter+1
        if ( iter > ITER_EM_MAX) then
          call stop_application( ' the EM algorithm for the estimation of haplotypes frequency does not converge ')
        end if
       do jhc=1,nb_haplo_complet        
          pb_haplo_complet(jhc)=phc(jhc)
       end do !jhc
      end do !while
        S=0.D0
      ! print *,'RACE       PB_HAPLO' 
       do jhc=1,nb_haplo_complet        
          S=S+pb_haplo_complet(jhc)
          !print *,trim(race_haplo_complet(jhc)), pb_haplo_complet(jhc)
       end do !jhc
       ! print *, 'Somme PROB_haplo_complet', S, nb_haplo_complet 

      deallocate (phc)
      deallocate (pht)

      end subroutine proba_haplo_complet
!!***

!!****f* m_qtlmap_haplotype_ldla/tri_haplo_complet
!! NAME
!!    tri_haplo_complet
!! DESCRIPTION
!!
!! NOTES
!! tri_haplo_complet elimine de la liste les haplotypes complets peu probables
!!
!! en entree, il lui faut
!!	nb_haplo_complet et tab_haplo_complet créés par la routine liste_haplo_complet du module m_qtlmap_haplotype_ldla
!!       haplo et nb_haplo créés par la routine set_allele_info_vector du module m_qtlmap_genotype
!! en sortie il fournit
!!	pb_haplo_complet recalcule en, eliminant les haplotypes peu probables
!!        name_haplo_complet le nom compacte des haplotypes complets
!! HISTORY
!!  correction BUG : leak memory 05/11/2010
!! SOURCE
     subroutine tri_haplo_complet(c,dx)
       integer       , intent(in) :: c
       real(kind=dp) , intent(in) :: dx
       integer  :: stat,lstr1,lstr2
       integer  jhc,i,idx,ipb,khr,iht,ir
       integer nb_possible
       double precision tphc
       logical une_race_rare
       character(kind=1) :: nm_race_rare
       real(kind=dp) :: spb_haplo_reduit
       real(kind=dp) :: pb,pbtot !cmo
! dim allocatable
     double precision          , dimension (:)    , allocatable    :: phc
     double precision          , dimension (:)    , allocatable    :: pbtot_haplo_complet !cmo
! allocation
      allocate (phc(nb_haplo_complet*NB_RACES),STAT=stat)
      call check_allocate(stat,'phc [m_qtlmap_haplotype_ldla:tri_haplo_complet]')

      if ( size(liste_haplo_reduit,1) < nb_haplo_complet*NB_RACES ) then
        deallocate (liste_haplo_reduit)
        allocate (liste_haplo_reduit(nb_haplo_complet*NB_RACES))
      end if

      allocate (pbtot_haplo_complet(NB_RACES),STAT=stat) !CMO
      call check_allocate(stat,'pbtot_haplo_complet [m_qtlmap_haplotype_ldla:tri_haplo_complet]') !cmo


      !correction BUG OFI : 14/09/2010
      if ( size (name_haplo_complet)<nb_haplo_complet*NB_RACES ) then
        deallocate(name_haplo_complet)
        allocate (name_haplo_complet(nb_haplo_complet*NB_RACES))
      end if

!
!  nom compacte des haplotypes mis dans name_haplo_complet
!
      name_haplo_complet=''

      do jhc=1,nb_haplo_complet
        lstr1=len(trim(get_pheno(c,tab_haplo_complet(jhc,1))))
        name_haplo_complet(jhc)(1:lstr1)=trim(get_pheno(c,tab_haplo_complet(jhc,1)))

        do i=2,longhap
          lstr2=len(trim(get_pheno(c,tab_haplo_complet(jhc,i))))
          name_haplo_complet(jhc)(lstr1+1:lstr1+lstr2)=trim(get_pheno(c,tab_haplo_complet(jhc,i)))
          lstr1=lstr1+lstr2
        end do !i

      end do ! jhc
!
!  comptage des haplotypes complets poossibles par haplotype trouvé
!
        do iht =1,nb_haplo
         nb_haplo_possible(iht)=0
         do jhc=1,nb_haplo_complet
           if(comp_haplo(iht,jhc)) nb_haplo_possible(iht)=nb_haplo_possible(iht)+1
         end do ! jhc
      end do !iht 

!!!CMO
   ! Recalcul de pb_haplo_complet(jhc): Calcul des freq haplotypique intrarace
        pbtot_haplo_complet=0.d0
        pbtot=0.d0
            do ir=1, nb_races
              do jhc=1,nb_haplo_complet
                    if (race_haplo_complet(jhc)==nom_race(ir)) pbtot_haplo_complet(ir)=pbtot_haplo_complet(ir)+pb_haplo_complet(jhc)
                enddo
                pbtot=pbtot+pbtot_haplo_complet(ir)
                !print *, 'PB_race',ir, pbtot_haplo_complet(ir)
             enddo
             !print *, 'PBtot', pbtot
!!!CMO

!  regroupement des haplotypes rares dans le paquet des donnees manquantes
!  on prepare le calcul des probabilites des haplotypes non rares (phc(jhc))
!  et on les compte (nb_haplo_reduit)
!
        phc=0.d0
        tphc=0.d0
        i=0
        nb_haplo_reduit=0
        liste_haplo_reduit=0
        spb_haplo_reduit=0.D0
        une_race_rare=.true.
        nm_race_rare='UNKNOWN'
        !print *,PROB_HAPLO_MIN
        do jhc=1,nb_haplo_complet
!!!CMO
             pb=0.d0
              do ir=1, nb_races
                if (race_haplo_complet(jhc)==nom_race(ir)) then
                    pb=pb_haplo_complet(jhc)
                    pb_haplo_complet(jhc)=(pb*pbtot)/(pbtot_haplo_complet(ir))
                    !print *, race_haplo_complet(jhc), 'AVANT ',pb, 'APRES ', pb_haplo_complet(jhc)
                endif
              enddo
!!!CMO
          if(pb_haplo_complet(jhc) > PROB_HAPLO_MIN) then
            phc(jhc)=pb_haplo_complet(jhc)
            tphc=tphc+phc(jhc)
            nb_haplo_reduit= nb_haplo_reduit+1
            liste_haplo_reduit(nb_haplo_reduit)=jhc
            spb_haplo_reduit=spb_haplo_reduit+phc(jhc)
          else
              
              if (race_haplo_complet(jhc).ne.'UNKNOWN') i=i+1 
              if (i==1) nm_race_rare=race_haplo_complet(jhc)  
              if(nm_race_rare.ne.'UNKNOWN'.and.race_haplo_complet(jhc).ne.nm_race_rare) &
                une_race_rare=.false.   
          end if
        end do ! jhc 
 !print *,'somme des p_haplo=',    spb_haplo_reduit , nb_haplo_complet
! 
!  arret si aucun haplotype possible
!
        if ( tphc == 0.d0) then
          return
          idx=int(100.*dx)
          ipb=int(100.*PROB_HAPLO_MIN)
          call stop_application(' No possible haplotypes found at location '//trim(str(idx))//&
                                ' cM with a threshold '//trim(str(ipb))//' %')
        end if
!
! allocations selon le nobre d'haplotypes non rare
! 
      if ( size(comp_haplo_reduit,1) < nb_haplo .or. size(comp_haplo_reduit,2) < nb_haplo_reduit+nb_races ) then
       deallocate (comp_haplo_reduit)
       allocate (comp_haplo_reduit(nb_haplo,nb_haplo_reduit+nb_races),STAT=stat)
       call check_allocate(stat,'comp_haplo_reduit [m_qtlmap_haplotype_ldla]')
      end if
 
      if ( size(pb_haplo_reduit,1) < nb_haplo_reduit+nb_races ) then
        deallocate (pb_haplo_reduit)
        allocate (pb_haplo_reduit(nb_haplo_reduit+nb_races),STAT=stat)
        call check_allocate(stat,'pb_haplo_reduit [m_qtlmap_haplotype_ldla]')
      end if

      if ( size(name_haplo_reduit,1) <  nb_haplo_reduit+nb_races ) then
        deallocate (name_haplo_reduit)
        allocate (name_haplo_reduit(nb_haplo_reduit+nb_races),STAT=stat)
        call check_allocate(stat,'name_haplo_reduit [m_qtlmap_haplotype_ldla]')
      end if

!
!  creation des probabilites et correspondances
!
       pb_haplo_reduit =0.d0
      race_haplo_reduit='UNKNOWN'

       do khr=1,nb_haplo_reduit
          pb_haplo_reduit(khr)=phc(liste_haplo_reduit(khr))
!         pb_haplo_reduit(khr)=phc(liste_haplo_reduit(khr))/tphc
          name_haplo_reduit(khr)=name_haplo_complet(liste_haplo_reduit(khr))
          race_haplo_reduit(khr)=race_haplo_complet(liste_haplo_reduit(khr))
       end do ! khr


      do iht =1,nb_haplo
         nb_possible=0
         do khr=1,nb_haplo_reduit
            comp_haplo_reduit(iht,khr)=comp_haplo(iht,liste_haplo_reduit(khr))
            if(comp_haplo_reduit(iht,khr)) nb_possible=nb_possible+1
         end do ! khr
         if(nb_possible < nb_haplo_possible(iht))comp_haplo_reduit(iht,nb_haplo_reduit+1)=.true.
      end do !iht    

      if(nb_haplo_reduit < nb_haplo_complet) then
         if (raceFileDefined) then
             do ir=1, nb_races
               spb_haplo_reduit=0.d0
               nb_haplo_reduit= nb_haplo_reduit+1
               do jhc=1,nb_haplo_complet
                 if ((race_haplo_complet(jhc)==nom_race(ir)) &
                 .and.(pb_haplo_complet(jhc).LT.PROB_HAPLO_MIN)) &
                  spb_haplo_reduit=spb_haplo_reduit+pb_haplo_complet(jhc)
                end do !jhc
               pb_haplo_reduit(nb_haplo_reduit)= spb_haplo_reduit
               name_haplo_reduit(nb_haplo_reduit)='RARE'
               race_haplo_reduit(nb_haplo_reduit)=nom_race(ir)

            enddo
         else
          nb_haplo_reduit= nb_haplo_reduit+1
          pb_haplo_reduit(nb_haplo_reduit)=1.d0-tphc
          name_haplo_reduit(nb_haplo_reduit)='RARE'
          if (une_race_rare) race_haplo_reduit(nb_haplo_reduit)=nm_race_rare
         end if
       end if

       nb_max_haplo=nb_haplo_reduit
 
!      write(nficout,*)  'Num_HAPLO   RACE    HAPLO    pb_HAPLO'
!      do i= 1, nb_haplo_reduit      
!         write(nficout,*) i,trim(race_haplo_reduit(i)),' ',&
 !                   trim(name_haplo_reduit(i)), &
!                    pb_haplo_reduit(i)
 !     enddo

       !  correctiopn BUG : leak memory 05/11/2010
       deallocate (phc)
       deallocate (pbtot_haplo_complet)
      end subroutine tri_haplo_complet
!!***

!!****f* m_qtlmap_haplotype_ldla/set_haplo_final
!! NAME
!!    set_haplo_final
!! DESCRIPTION
!!
!! NOTES
!! set_haplo_final met au propre les listes de correspondances des haplotypes paternels et maternels
!!  num_haplo_pere(ip,j,l)  pour le pere ip, chromosome j et haplotype complet de la liste reduite possible l
!!   (avec = 1,... nb_gam_pere(ip,j))
!!  num_haplo_mere(geno,j,l)  pour la combinaison mere x phase geno, chromosome j et haplotype complet de la liste
!!    reduite possible l  (avec = 1,... nb_gam_mere(geno,j))
!!  num_haplo_desc(kd,j,l)  pour le descendant kd, chromosome j et haplotype complet de la liste
!!    reduite possible l  (avec = 1,... nb_gam_desc(kd))
!!
!!  Utilise np,nm,ndm,estfem,repfem de m_qtlmap_data
!!  utilise nb_haplo_reduit et comp_haplo_reduit  de m_qtlmap_haplotype_ldla (tri_haplo_complet)
!!  utilise ngenom de m_qtlmap_haplotpe_data
!!  utilise presentg
!!  utilise nb_gam
!! SOURCE
     subroutine set_haplo_final(c,n,hsire,hdam)

      logical, intent (in) :: hsire,hdam
      integer, intent (in) :: n,c
      integer  :: stat
! divers
      integer ip,j,j_haplo,k_haplo,jm,geno,kd,j_gam
      integer  :: nb_gam_t,i_gam,value_alloc,value_alloc_2
      real (kind=dp)  :: su

      if ( size(num_haplo_pere,3) <  nb_haplo_complet ) then
        deallocate (num_haplo_pere)
        allocate (num_haplo_pere(np,2,nb_haplo_complet),STAT=stat)
        call check_allocate(stat,'num_haplo_pere [m_qtlmap_haplotype_ldla]')
        end if
      num_haplo_pere=0

      if ( size(num_haplo_mere,3) <  nb_haplo_complet ) then
        deallocate (num_haplo_mere)
        allocate (num_haplo_mere(maxval(ngenom(:,nm+1)),2,nb_haplo_complet),STAT=stat)
        call check_allocate(stat,'num_haplo_mere [m_qtlmap_haplotype_ldla]')
      end if

      num_haplo_mere = 0
      value_alloc=nb_haplo_reduit*maxval(nb_gam)
      value_alloc_2=maxval(nall)**(2*longhap)+1

      if ( size(num_haplo_desc,2) <  value_alloc ) then
        deallocate (num_haplo_desc)
        allocate (num_haplo_desc(nd,value_alloc),STAT=stat)
        call check_allocate(stat,'num_haplo_desc [m_qtlmap_haplotype_ldla]')
      end if

      num_haplo_desc = 0  

      if ( size(pb_haplo_desc,2) <  value_alloc_2) then
        deallocate (pb_haplo_desc)
        allocate (pb_haplo_desc(nd,value_alloc_2),STAT=stat)
        call check_allocate(stat,'pb_haplo_desc [m_qtlmap_haplotype_ldla]')
      end if

      pb_haplo_desc = 0.d0

       j_haplo=1

!
!  pour les peres
!
       nb_gam_pere=0
       if(hsire)then

       do ip=1,np
         do j=1,2
           j_haplo=j_haplo+1
           do k_haplo=1,nb_haplo_reduit
             if(comp_haplo_reduit(j_haplo,k_haplo)) then
               nb_gam_pere(ip,j)=nb_gam_pere(ip,j)+1
               num_haplo_pere(ip,j,nb_gam_pere(ip,j))= k_haplo
             end if
           end do !k_haplo
          end do !j
        end do !ip

       end if ! opt_model(hsire)
!
!  pour les meres 
!
       nb_gam_mere=0
       nb_gam_desc=0
       if(hdam)then

       do jm=1,nm
         if(estfem(repfem(jm)))then
!
!  meres phasees
!
           do geno=ngenom(c,jm)+1,ngenom(c,jm+1)
             do j=1,2
               j_haplo=j_haplo+1
                 do k_haplo=1,nb_haplo_reduit
                   if(comp_haplo_reduit(j_haplo,k_haplo)) then
                     nb_gam_mere(geno,j)=nb_gam_mere(geno,j)+1
                     num_haplo_mere(geno,j,nb_gam_mere(geno,j))= k_haplo
                   end if
                 end do !k_haplo
             end do !j
           end do !geno
         else
! 
!   non phasees
!           do kd=ndm(jm)+1,ndm(jm+1)
!             if(presentg(c,kd))then
!             do j_gam=1,nb_gam(kd)
!               j_haplo=j_haplo+1
!                 do k_haplo=1,nb_haplo_reduit
!                   if(comp_haplo_reduit(j_haplo,k_haplo)) then
!                     nb_gam_desc(kd,j_gam)=nb_gam_desc(kd,j_gam)+1
!                     num_haplo_desc(kd,j_gam,nb_gam_desc(kd,j_gam))= k_haplo
!                   end if
!                 end do !k_haplo
!             end do !j_gam
!             end if
!            end do !kd

           do kd=ndm(jm)+1,ndm(jm+1)
             if(presentg(c,kd))then
               do j_gam=1,nb_gam(kd)
                 j_haplo=j_haplo+1
                 nb_gam_t=nb_gam_desc(kd)+1
                 do k_haplo=1,nb_haplo_reduit
                   if(comp_haplo_reduit(j_haplo,k_haplo)) then
                     nb_gam_desc(kd)=nb_gam_desc(kd)+1
                     num_haplo_desc(kd,nb_gam_desc(kd))=k_haplo
                   end if
                 end do  !k
                 su = dble(sum(pb_haplo_reduit(num_haplo_desc(kd,nb_gam_t:nb_gam_desc(kd)))))
                 if ( su == 0 ) cycle
                 do i_gam=nb_gam_t,nb_gam_desc(kd)
 !                   pb_haplo_desc(kd,i_gam)  = pb_haplo_desc(kd,i_gam)  &
 !                       +prob_gam(kd,j_gam) * (pb_haplo_reduit(num_haplo_desc(kd,i_gam)) /su)
                   pb_haplo_desc(kd,num_haplo_desc(kd,i_gam))  =  &
                         pb_haplo_desc(kd,num_haplo_desc(kd,i_gam))  &
                        +prob_gam(kd,j_gam) * (pb_haplo_reduit(num_haplo_desc(kd,i_gam)) /su)
                 end do !i_gam
               end do !j_gam
             end if !presentg(c,kd)
           end do !kd


         end if ! estfem
       end do !jm
       end if ! opt_model (hdam)

     end  subroutine set_haplo_final
!!***

!!****f* m_qtlmap_haplotype_ldla/local_gamete
!! NAME
!!    local_gamete
!! DESCRIPTION
!!
!! NOTES
!!  local_gamete trouve les gametes parentaux transmis par ses parents (ip et jm)
!!  pour une phase geno de la mere
!!  a un individu particulier (kd)
!!  autour d'un point precis defini par le chromosome (c) et la position (dx)
!!  ainsi que le nombre de marqueurs l'entourant (longhap)
!!
!!  en sortie, local_gamete fournit
!!	nb_gam = le nombre de couples de gametes parentaux (2 ** nb indeterminations)
!!	gamete(1,lk,j_gam) l allele paternel au lk eme marqueur pour le j_gam eme gamete
!!	gamete(2,lk,j_gam) l allele maternel au lk eme marqueur pour le j_gam eme gamete
!!   prob_gam(j_gam) la probabilite de cet allele
!!
!!  Utilise posi,nmk,nmanque,correp,corred,correm,pheno     de m_qtlmap_data
!!  Utilise genotyp,genotypm                                de m_qtlmap_haplotype_data
!!
!!  HISTORY
!!    correction BUG : leak memory 05/11/2010
!! SOURCE
   subroutine local_gamete(ip,jm,geno,kd,c,dx)
      integer , intent(in) ::ip,jm,geno,kd,c
      real (kind=dp), intent(in) :: dx

! divers
      integer lk,lkleft,lkright,l,llk
      integer left_known_sire,left_known_dam,right_known_sire,right_known_dam
      integer lks,lkd,rks,rkd
      integer isex,igam
      integer i,j,k,idup
      integer ngamkept,nb_inc
      integer value_alloc
      logical known_sire_left,known_dam_left,known_sire_right,known_dam_right,manquant
      logical ksr,ksl,kdr,kdl
      integer (kind=KIND_PHENO) :: allele(2),unknown
      integer :: orig(2)
      real(kind=dp) recm,recf,probtot
      integer  :: stat

! dimension allocatable
      integer                   , dimension (:,:)    , allocatable   :: base
      integer(kind=KIND_PHENO)  , dimension (:,:,:)  , allocatable   :: origine
      integer                   , dimension (:,:)    , allocatable   :: tabdup
! allocation
!
      allocate (base(2,nmk(c)),STAT=stat)
      call check_allocate(stat,'base [m_qtlmap_haplotype_LD]')

      value_alloc= 2**(2*longhap)

      allocate (origine(2,nmk(c),value_alloc),STAT=stat)
      call check_allocate(stat,'origine [m_qtlmap_haplotype_LD]')

      allocate (tabdup(value_alloc,2*longhap),STAT=stat)
      call check_allocate(stat,'tabdup[m_qtlmap_haplotype_LD]')
      unknown=0

      gamete=nmanque

!  la position testee doit etre compatible avec la longueur de l'haplotype
!

      if ( longhap/2 > 0 ) then
       if((dx < posi(c,longhap/2)) .or. (dx > posi(c,nmk(c)-longhap/2+1)))then
       call stop_application("local_gamete : incompatibilty between haplotype length and tested position.")
       end if
      end if
!
! on cherche le marqueur flanquant a gauche la position dx
! 
       lk=1
       do while (dx > posi(c,lk))
         lk=lk+1
       end do
      lk=lk-1
!
! lkleft et lkright seront les numeros des marqueurs bornant l haplotype a determiner
!
       lkleft  =lk-longhap/2+1
       if(lk <= longhap/2) lkleft=1
       lkright =lk+longhap/2
       if(lk >= nmk(c)-longhap/2) lkright=nmk(c)
!
!  du fait des incertitudes 
!      (pere et descendant heterozygotes identiques si la mere n est pas marquee
!       deux parents et descendants heterozygotes identiques si la mere est marquee)
!     plusieurs gametes peuvent avoir forme un descendant
!
!  nb_gam = 2**(nb d'incertitudes) est le nombre de gametes possibles
!  gamete(1,lk,j_gam) est le nom de l'allele au i eme marqueur transmis par le pere 
!    (par la mere pour gamete(2,lk,j_gam))
!     pour la j_gam eme configuration gametique
!  origine(1,lk,j_gam) en est l origine grand paternel 
!     (0 pour inconnue, 1 si grand pere, 2 si grand mere, 3 le pere est homozygote, 4 si manquant)
!   (grand maternelle pour origine(2,lk))
!
!  construction des gametes correspondant a l haplotype
!  pour chaque marqueur de l'intervalle, on cherche (avec point_gamete) le nom des alleles recus
!  de chacun des parents, ainsi que leur origine grand parentale (mis dans base)
! base vaut 1 si origine = grand pere; 2 = grand mere ; 
!           0 = indetermine (duo ou triplet heterozygote identique); 3= parent homozygote, 4= manquant
!
       origine =0
       base=0

      manquant=.false.
       do lk = lkleft,lkright
         call point_gamete(ip,jm,geno,kd,c,lk,allele,base(:,lk))
         if(allele(1) == nmanque .or. allele(2) == nmanque) manquant=.true.
          origine(1,lk,1)=base(1,lk)
          origine(2,lk,1)=base(2,lk)
       end do !lk
 
!
!  je mets a manque total les haplotypes compremnant une donnee manquante
!
     if(manquant) then
       nb_gam(kd)=1
       prob_gam(kd,1)=1.d0

       llk=0
       do lk = lkleft,lkright
         llk=llk+1
         do isex=1,2
           gamete(isex,llk,1)=nmanque
         end do ! isex
       end do !lk

       deallocate (base)
       deallocate (origine)
       deallocate (tabdup)

       return

      end if
!
! il y a 2**(nombre d'origines inconnues) gametes possibles dont on doit calculer les probabilites
!
! quand les marqueurs flanquants sont d'origine inconnue, il faut rechercher une information 
! a l exterieur de l'haplotype
!
!  vers la gauche
!
       left_known_sire=0
       left_known_dam=0
       known_sire_left=.false.
       if(base(1,lkleft) ==1 .or.base(1,lkleft) == 2) then
         known_sire_left=.true.
         left_known_sire=lkleft
       end if
       known_dam_left=.false.
       if(base(2,lkleft) ==1 .or.base(2,lkleft) == 2) then
         known_dam_left=.true.
         left_known_dam=lkleft
       end if

       if(.not.manquant)then
       if(estfem(repfem(jm))) then

         if(.not. (known_sire_left .and. known_dam_left))  then
           do lk=lkleft-1,1,-1 
             call point_gamete(ip,jm,geno,kd,c,lk,allele,base(:,lk))

          if(.not. known_sire_left .and. (base(1,lk) == 1 .or. base(1,lk) == 2)) then
               known_sire_left=.true.
               left_known_sire=lk 
               origine(1,lk,1)=base(1,lk)
             end if
             if(.not. known_dam_left .and. (base(2,lk) == 1 .or. base(2,lk) == 2)) then
               known_dam_left=.true.
               left_known_dam=lk 
               origine(2,lk,1)=base(2,lk)
             end if

             if(known_sire_left .and. known_dam_left) exit
           end do !lk
         end if  
  
       else  

         if(.not.known_sire_left)  then
           do lk=lkleft-1,1,-1 
            call point_gamete(ip,jm,geno,kd,c,lk,allele,base(:,lk))

             if(.not. known_sire_left .and. (base(1,lk) == 1 .or. base(1,lk) == 2)) then
               known_sire_left=.true.
               left_known_sire=lk 
               origine(1,lk,1)=base(1,lk)
             end if          
             if(known_sire_left) exit
           end do !lk
         end if

       end if
       end if
!
!  vers la droite
!
       right_known_sire=0
       right_known_dam=0
       known_sire_right=.false.
       if(base(1,lkright) ==1 .or.base(1,lkright) == 2) then
         known_sire_right=.true.
         right_known_sire=lkright
       end if
       known_dam_right=.false.
       if(base(2,lkright) ==1 .or.base(2,lkright) == 2) then
         known_dam_right=.true.
         right_known_dam=lkright
       end if

       if(.not.manquant) then
       if(estfem(repfem(jm))) then

         if(.not. (known_sire_right .and. known_dam_right))  then

           do lk=lkright+1,nmk(c) 

             call point_gamete(ip,jm,geno,kd,c,lk,allele,base(:,lk))


             if(.not. known_sire_right .and. (base(1,lk) == 1 .or. base(1,lk) == 2)) then
               known_sire_right=.true.
               right_known_sire=lk
               origine(1,lk,1)=base(1,lk)
             end if
             if(.not. known_dam_right .and. (base(2,lk) == 1 .or. base(2,lk) == 2)) then
               known_dam_right=.true.
               right_known_dam=lk
               origine(2,lk,1)=base(2,lk)
             end if

             if(known_sire_right .and. known_dam_right) exit
           end do !lk
         end if
       else

         if(.not.known_sire_right)  then
           do lk=lkright+1,nmk(c)
             call point_gamete(ip,jm,geno,kd,c,lk,allele, base(:,lk))

             if(.not. known_sire_right .and. ( base(1,lk) == 1 .or.  base(1,lk) == 2)) then
               known_sire_right=.true.
               right_known_sire=lk
               origine(1,lk,1)= base(1,lk)
             end if
             if(known_sire_right) exit
           end do !lk
         end if
       end if
       end if
!
! Construction de la liste des gametes possibles
! en parcourant les marqueurs le long de l haplotype et en dedoublant la liste 
! a chaque fois qu'une incertitude (base =0) est rencontree
! nb_inc est le nombre d'incertitudes
! nb_gam(kd) le nombre de gametes possibles pour kd (nb_gam(kd)=2**nb_inc dans un premier temps)
!
        nb_inc=0

        do lk = lkleft,lkright
          do isex=1,2
            if(base(isex,lk) == 0) nb_inc=nb_inc + 1
          end do !isex
        end do !lk
        nb_gam(kd)=2**nb_inc
!
!tabdup contient la liste des vecteurs d'origines possibles pour les nb_gam(kd) incertitudes
!
       if(nb_inc > 0) then
         tabdup(1,nb_inc)=1
         tabdup(2,nb_inc)=2
         do i=2,nb_inc
           do j=1,2**(i-1)
             tabdup(j,nb_inc-i+1)=1
             tabdup(2**(i-1)+j,nb_inc-i+1)=2
             do k=1,i-1
               tabdup(2**(i-1)+j,nb_inc-k+1)=tabdup(j,nb_inc-k+1)
             end do !k
           end do !j
         end do !i

         do igam=1,nb_gam(kd)
           idup=0
           do lk = lkleft,lkright
             do isex=1,2
               origine(isex,lk,igam)=origine(isex,lk,1)
               if(base(isex,lk) == 0) then
                 idup=idup+1
                 origine(isex,lk,igam)=tabdup(igam,idup)
               end if
             end do ! isex
           end do ! lk
           if(known_sire_left)  origine(1,left_known_sire,igam) =origine(1,left_known_sire,1)
           if(known_sire_right) origine(1,right_known_sire,igam)=origine(1,right_known_sire,1)
           if(known_dam_left)   origine(2,left_known_dam,igam) = origine(2,left_known_dam,1)
           if(known_dam_right)  origine(2,right_known_dam,igam)= origine(2,right_known_dam,1)
         end do ! igam

       end if !nb_gam


     ksr=known_sire_right
     ksl=known_sire_left
     kdr=known_dam_right
     kdl=known_dam_left
     lks=left_known_sire 
     rks=right_known_sire 
     lkd=left_known_dam
     rkd=right_known_dam 
  
!
!  calcul des probabilites des gametes
!
           do igam=1,nb_gam(kd)
            prob_gam(kd,igam)=1.d0
            known_sire_right=ksr
            known_sire_left=ksl
            known_dam_right=kdr
            known_dam_left=kdl
            left_known_sire=lks
            right_known_sire=rks 
            left_known_dam=lkd
            right_known_dam=rkd 

            do lk=lkleft,lkright

             if(origine(1,lk,igam) < 3) then
               if(known_sire_left) then
                 recm=xaldane(posim(c,lk)-posim(c,left_known_sire))
                 if(origine(1,lk,igam) ==  origine(1,left_known_sire,igam)) then
                   prob_gam(kd,igam)=prob_gam(kd,igam) * (1.d0 -recm)
                 else
                   prob_gam(kd,igam)=prob_gam(kd,igam) * recm
                 end if
               else
                 known_sire_left=.true.
               end if
               left_known_sire=lk
             end if

             if(geno /=0 .and. origine(2,lk,igam) < 3) then
               if(known_dam_left) then
                 recf=xaldane(posif(c,lk)-posif(c,left_known_dam))
                 if(origine(2,lk,igam) ==  origine(2,left_known_dam,igam)) then
                   prob_gam(kd,igam)=prob_gam(kd,igam) * (1.d0 -recf)
                 else
                   prob_gam(kd,igam)=prob_gam(kd,igam) * recf
                 end if
               else
                 known_dam_left=.true.
               end if
               left_known_dam=lk
             end if

            end do ! lk
!
! bout du segment
             if(known_sire_right .and. right_known_sire > lkright) then
              recm=xaldane(posim(c,right_known_sire)-posim(c,lkright))
               if(origine(1,lkright,igam) ==  origine(1,right_known_sire,igam)) then
                 prob_gam(kd,igam)=prob_gam(kd,igam) * (1.d0 -recm)
               else
                 prob_gam(kd,igam)=prob_gam(kd,igam) * recm
               end if
              end if
             if(geno /= 0 .and. known_dam_right .and. right_known_dam > lkright) then
              recf=xaldane(posif(c,right_known_dam)-posif(c,lkright))
               if(origine(2,lkright,igam) ==  origine(2,right_known_dam,igam)) then
                 prob_gam(kd,igam)=prob_gam(kd,igam) * (1.d0 -recf)
               else
                 prob_gam(kd,igam)=prob_gam(kd,igam) * recf
               end if
              end if

          end do !igam
 
!  au cas ou, standardisation des probabilites
!
       probtot=0.d0
       do igam=1,nb_gam(kd)
        probtot=probtot+prob_gam(kd,igam)
       end do
       do igam=1,nb_gam(kd)
        prob_gam(kd,igam)=prob_gam(kd,igam)/probtot
       end do
!
!  on enrichir les informations sur les gametes pour chacune des configurations possibles
!
      gamete=nmanque
      do igam=1,nb_gam(kd)
        llk=0
        do lk=lkleft,lkright
          llk=llk+1
          if(origine(1,lk,igam) == 4) gamete(1,llk,igam) = nmanque
          if(origine(1,lk,igam) == 3) gamete(1,llk,igam) = genotyp(c,lk,correp(ip),1)  
          if(origine(1,lk,igam) == 2) gamete(1,llk,igam) = genotyp(c,lk,correp(ip),2)  
          if(origine(1,lk,igam) == 1) gamete(1,llk,igam) = genotyp(c,lk,correp(ip),1)  


          if(origine(1,lk,igam) <= 3) then
       
             if(gamete(1,llk,igam) == pheno(c,lk,corred(kd),1)) then
                  gamete(2,llk,igam) = pheno(c,lk,corred(kd),2)
             else
                  gamete(2,llk,igam) = pheno(c,lk,corred(kd),1)
             end if
          end if
        end do ! lk
      end do !igam
!
!
! elimination des cas improbables
!
     ngamkept=0
     do igam=1,nb_gam(kd)
       if(prob_gam(kd,igam) >= prob_seuil_gam)then
          ngamkept=ngamkept+1
          prob_gam(kd,ngamkept)=prob_gam(kd,igam)
          do llk=1,longhap
            gamete(1,llk,ngamkept)=gamete(1,llk,igam)
            gamete(2,llk,ngamkept)=gamete(2,llk,igam)
          end do !i
       end if
     end do !igam
!
! standardisation
!
       probtot=0.d0
       do igam=1,ngamkept
        probtot=probtot+prob_gam(kd,igam)
       end do
       do igam=1,ngamkept
        prob_gam(kd,igam)=prob_gam(kd,igam)/probtot
       end do

     nb_gam(kd)=ngamkept

       ! correctiopn BUG : leak memory 05/11/2010
       deallocate (base)
       deallocate (origine)
       deallocate (tabdup)
       end subroutine local_gamete
!!***

!!****f* m_qtlmap_haplotype_ldla/point_gamete
!! NAME
!!    point_gamete
!! DESCRIPTION
!!
!! NOTES
!!
!! point_gamete determine le nom des alleles (allele) au marqueur lk du chromosome c
!!  recus par les descendant kd du pere ip et de la mere jm  affectee de la phase geno
!!  ainsi que leur origine grand parentale (orig)
!! allele(1) est l'allele recu du pere, allele(2) de la mere
!! allele(1)=allele(2)=0 si le duo ou le triplet est heterozygte identique
!! allele(1)=allele(2)=nmanque si le phenotype du descednant et ou du pere sont manquants
!! orig(1) vaut 1 si origine = grand pere paternel ; 2 = grand mere paternelle;
!!           0 = indetermine (duo ou triplet heterozygote identique); 3= pere homozygote
!! orig(2) est le symetrique pour l'origne maternelle
!!
!! Utilise nmanque,correp,pheno,corred,correm		de m_qtlmap_data
!! Utilise genotyp,genotypm				de m_qtlmap_genotype_data
!! SOURCE
       subroutine point_gamete(ip,jm,geno,kd,c,lk,allele,orig)


     integer(kind=KIND_PHENO) :: allele(2),unknown

     integer :: orig(2)
     integer , intent(in) :: ip,jm,geno,kd,c,lk
     logical found

        allele(1)=nmanque
        allele(2)=nmanque
        unknown=0
        found=.false.
!  le genotype du pere et le phenotype du descendant ne doivent pas etre manquants
!
         if(genotyp(c,lk,correp(ip),1) /= nmanque .and. pheno(c,lk,corred(kd),1) /= nmanque) then  !1
!
!  cas du pere homozygote
!
         if(genotyp(c,lk,correp(ip),1) == genotyp(c,lk,correp(ip),2)) then  !2

           allele(1)=genotyp(c,lk,correp(ip),1)
           if(pheno(c,lk,corred(kd),1) == allele(1)) then
             allele(2)=pheno(c,lk,corred(kd),2)
           else
             allele(2)=pheno(c,lk,corred(kd),1)
           end if
           found=.true.
!
! pere heterozygote

         else
!    descendant homozygote
!
           if(pheno(c,lk,corred(kd),1) == pheno(c,lk,corred(kd),2)) then !3
             allele(1)=pheno(c,lk,corred(kd),1)
             allele(2)=pheno(c,lk,corred(kd),2)
             found=.true.
           end if

! descendant heterozygote
!  son premier allele ne pouvant pas venir du pere
!
           if(pheno(c,lk,corred(kd),1) /=   genotyp(c,lk,correp(ip),1) .and. &
                  pheno(c,lk,corred(kd),1) /=   genotyp(c,lk,correp(ip),2)) then    !4
              allele(1)=pheno(c,lk,corred(kd),2)
              allele(2)=pheno(c,lk,corred(kd),1)
              found=.true.
           end if
!
! descendant heterozygote
!  son deuxieme allele ne pouvant pas venir du pere
!
           if(pheno(c,lk,corred(kd),2) /=   genotyp(c,lk,correp(ip),1) .and. &
                  pheno(c,lk,corred(kd),2) /=   genotyp(c,lk,correp(ip),2)) then      !5
               allele(1)=pheno(c,lk,corred(kd),1)
               allele(2)=pheno(c,lk,corred(kd),2)
               found=.true.
           end if
!
! incertitude pere - descendant, la mere etant connue
!
           if (.not. found ) then   !6
            allele(1)=unknown
            allele(2)=unknown
            if ( pheno(c,lk,correm(jm),1) /= nmanque ) then   !6
!
!  cas de la mere homozygote
!
             if(pheno(c,lk,correm(jm),1) == pheno(c,lk,correm(jm),2)) then  !7

               allele(2)=pheno(c,lk,correm(jm),1)
               if(pheno(c,lk,corred(kd),1) == allele(2)) then
                allele(1)=pheno(c,lk,corred(kd),2)
               else
                 allele(1)=pheno(c,lk,corred(kd),1)
               end if
               found=.true.
             end if
!
! mere heterozygote
! descendant heterozygote
!  son premier allele ne pouvant pas venir de la mere
!
             if(pheno(c,lk,corred(kd),1) /=   pheno(c,lk,correm(jm),1) .and. &
                  pheno(c,lk,corred(kd),1) /=   pheno(c,lk,correm(jm),2)) then  !8
              allele(1)=pheno(c,lk,corred(kd),1)
              allele(2)=pheno(c,lk,corred(kd),2)
              found=.true.
             end if
!
! descendant heterozygote
!  son deuxieme allele ne pouvant pas venir de la mere
!
             if(pheno(c,lk,corred(kd),2) /=   pheno(c,lk,correm(jm),1) .and. &
                  pheno(c,lk,corred(kd),2) /=   pheno(c,lk,correm(jm),2)) then  !9
               allele(1)=pheno(c,lk,corred(kd),2)
               allele(2)=pheno(c,lk,corred(kd),1)
               found=.true.
             end if

         end if
         end if
         end if
         end if

!
!  on peut alors determiner les origines grands parentales des gametes recus par kd
!
!  0 = indetermine (le descendant est heterozygote comme ses parents)
!  3 = indeterminable (les parents sont homozygotes )
!  4 =  donnees manquantes)
!
         orig(1)=0
         if(genotyp(c,lk,correp(ip),1) == nmanque .or. pheno(c,lk,corred(kd),1) == nmanque) orig(1) =4 
         if(orig(1) == 0 .and. (genotyp(c,lk,correp(ip),1) == genotyp(c,lk,correp(ip),2))) orig(1)=3
         if(orig(1) == 0 .and. allele(1) /= unknown) then
             if(allele(1) == genotyp(c,lk,correp(ip),1)) then
               orig(1)=1
             else
               orig(1)=2
             end if
         end if

         orig(2)=0
         if(geno /=  0) then
!
! cas ou la mere est phasee (orig correspond aux origines grand parentales)
!
          if(genotypm(c,lk,geno,1) == nmanque .or. pheno(c,lk,corred(kd),1) == nmanque) orig(2)= 4
          if(orig(2) == 0 .and. (genotypm(c,lk,geno,1) == genotypm(c,lk,geno,2))) orig(2)=3
          if(orig(2) == 0 .and. allele(2) /= unknown) then
                if(allele(2) == genotypm(c,lk,geno,1)) then
                  orig(2)=1
                else
                  orig(2)=2
                end if
           end if
         else
!
! cas ou la mere n'est pas phasee (orig correspond a l'ordre de lecture des phenotypes)
!
          if(pheno(c,lk,correm(jm),1) == nmanque .or. pheno(c,lk,corred(kd),1) == nmanque) orig(2)= 4
          if(orig(2) == 0 .and. (pheno(c,lk,correm(jm),1) == pheno(c,lk,correm(jm),2))) orig(2)=3
          if(orig(2) == 0 .and. allele(2) /= unknown) then
                if(allele(2) == pheno(c,lk,correm(jm),1)) then
                 orig(2)=1
                  else
                orig(2)=2
                end if
              end if
          end if
      end subroutine point_gamete
!!***

!!****f* m_qtlmap_haplotype_ldla/liste_gamete
!! NAME
!!    liste_gamete
!! DESCRIPTION
!!
!! NOTES
!!
!! liste_gamete trouve les gametes du chromosome c ayant forme
!! chacun des descendant
!!
!! Utilise nmk,np,posi,correp,nm,repfem,correm,estfem    de m_qtlmap_data
!! Utilise genotyp,ngenom,genotypm                       de m_qtlmap_haplotype_data
!! SOURCE
       subroutine liste_gamete(c)

!
      integer       , intent(in) :: c

      integer ip,jm,kd,geno,i,lk!,indtab(10000)

      geno=0
!
        do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            if (presentg(c,kd)) call global_gamete(ip,jm,geno,kd,c)

!            do i=1,2
!              do lk=1,nmk(c)
!                indtab(lk)=0
!                if(tab_IBS(kd,lk,i) /= 0)indtab(lk)=tab_IBS(kd,lk,i)- VAL_MIN_INDEX_PHENO
!              end do ! lk
     !       end do ! i
          end do !kd
        end do !jm
      end do !ip
!

         end subroutine liste_gamete
!!***

!!****f* m_qtlmap_haplotype_ldla/global_gamete
!! NAME
!!    global_gamete
!! DESCRIPTION
!!
!! NOTES
!!  global_gamete trouve les gametes parentaux transmis par ses parents (ip et jm)
!!  pour une phase geno de la mere
!!  a un individu particulier (kd)
!!  sur l'ensemble du chromosome (c)
!!  les doubles recombinaisons entre marqueurs informatifs flanquants sont supposees absentes
!!
!!  en sortie, global_gamete fournit
!!	tab_IBS(kd,lk,i) pour lk=1,nmk(c) et i=1 (gamete paternel) et 2 gamete maternel)
!!
!!
!!  Utilise nmk,nmanque,correp,corred,correm,pheno     de m_qtlmap_data
!!  Utilise genotyp                                    de m_qtlmap_haplotype_data
!!
!! HYSTORY
!!   correction BUG : leak memory 05/11/2010
!! SOURCE
         subroutine global_gamete(ip,jm,geno,kd,c)
      integer , intent(in) ::ip,jm,geno,kd,c

! divers
      integer lk,llk
      integer (kind=KIND_PHENO) orig_left,orig_right
      logical found
      integer (kind=KIND_PHENO) :: allele(2)
      integer :: orig(2)
      integer  :: stat
      integer (kind=KIND_PHENO) :: unknown=0
! dimension allocatable
      integer(kind=KIND_PHENO)  , dimension (:)  , allocatable   :: origine
! allocation
!
      allocate (origine(maxval(nmk)),STAT=stat)
      call check_allocate(stat,'origine [m_qtlmap_haplotype_LD]')

       tab_IBS(kd,:,:)=0

       do lk = 1,nmk(c)
!
!  pour chaque marqueur de l'intervalle, on cherche (avec point_gamete) le nom des alleles recus
!  de chacun des parents (allele), ainsi que leur origine grand parentale
!  orig vaut 1 si origine = grand pere; 2 = grand mere ;
!            0 = indetermine (duo ou triplet heterozygote identique); 3= parent homozygote, 4= manquant
!
         call point_gamete(ip,jm,geno,kd,c,lk,allele,orig)
         origine(lk)=orig(1)
         if(origine(lk) ==1 .or. origine(lk) ==2 .or. origine(lk) == 3) tab_IBS(kd,lk,1)=allele(1)
!
! si l'allele paternel est indermine on regarde ce qui s'est passe a gauche et a droite
         if(origine(lk) == 0) then

             orig_left=0
             found=.false.
             llk=lk
             do while((.not. found ).and. llk > 1)
               llk=llk-1
               if(origine(llk) ==1 .or. origine(llk) ==2 )  then
                  found=.true.
                  orig_left=origine(llk)
               end if
             end do ! found
             orig_right=0
             found=.false.
             llk=lk
             do while((.not. found ) .and. llk < nmk(c))
               llk=llk+1
               call point_gamete(ip,jm,geno,kd,c,llk,allele,orig)
               if(orig(1) ==1 .or. orig(1) ==2 )  then
                  found=.true.
                  orig_right=orig(1)
               end if
             end do ! found

             if(orig_left == orig_right) tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),orig_left)
             !if(orig_left == 0)          tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),orig_right)    !!ATTENTION orig_right peut valoir =0
             !if(orig_right == 0)         tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),orig_left)
             if( ((orig_left == 0).or.(orig_left == 4) ) .and. ( (orig_right /= 0) .and. (orig_right /= 4)) )  then
               if ( orig_right /= 3 ) then
                 tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),orig_right)
               else
                 tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),1)
               end if
             end if

             if( ((orig_right == 0).or.(orig_right == 4) ) .and. ( (orig_left /= 0) .and. (orig_left /= 4)) )  then
               if ( orig_left /= 3 ) then
                 tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),orig_left)
               else
                 tab_IBS(kd,lk,1)=genotyp(c,lk,correp(ip),1)
               end if
             end if

             !!*** pas de calcul de proba....a voir ***
             !! si orig_right et orig_left inconnu (0 ou 4) alors tab_IBS pas initialise...

         end if
!
!  connaissant l'allele recu du pre on en deduit celui recu de la mere
!
!         if(pheno(c,lk,corred(kd),1) == pheno(c,lk,corred(kd),2)) then
!           tab_IBS(kd,lk,2)=pheno(c,lk,corred(kd),1)
!         else
!           if(pheno(c,lk,corred(kd),1) == tab_IBS(kd,lk,1)) tab_IBS(kd,lk,2)=pheno(c,lk,corred(kd),2)
!           if(pheno(c,lk,corred(kd),2) == tab_IBS(kd,lk,1)) tab_IBS(kd,lk,2)=pheno(c,lk,corred(kd),1)
!         end if

           if(pheno(c,lk,corred(kd),1) == tab_IBS(kd,lk,1)) then
             tab_IBS(kd,lk,2)=pheno(c,lk,corred(kd),2)
           else
             tab_IBS(kd,lk,2)=pheno(c,lk,corred(kd),1)
           end if

       end do !lk

       ! correctiopn BUG : leak memory 05/11/2010
       deallocate (origine)
       end subroutine global_gamete
!!***

!!****f* m_qtlmap_haplotype_ldla/liste_shared_haplo
!! NAME
!!    liste_shared_haplo
!! DESCRIPTION
!!
!! NOTES
!!
!! liste_shared_haplo  determine si un descendant a recu de sa mere , autour de la positon dx,
!! un allele ibs a un de ceux de son pere, et sur quelle longueur
!! l'effet haplotypique sera ajoute au modle de description des performances si cette longueur
!! est superieure a LONG_MIN_IBS
!!
!! Utilise nmk,np,posi,correp,nm                         de m_qtlmap_data
!! Utilise genotyp                                       de m_qtlmap_haplotype_data
!! SOURCE
       subroutine liste_shared_haplo(c,dx,n)

!
      integer       , intent(in) :: c
      real(kind=dp) , intent(in) :: dx
      integer, intent (in) :: n

      integer lkleft,ip,jm,kd,jp,iall,icount,icount_max,lk,stat

      if (allocated(shared_haplo)) deallocate(shared_haplo)
      allocate (shared_haplo(nd),STAT=stat)
      call check_allocate(stat,'shared_haplo [m_qtlmap_haplotype_LD]')


!  la position testee doit etre entre les bornes du chromsome
!
      if((dx < posi(c,1)) .or. (dx > posi(c,nmk(c)))) then
        call stop_application("liste_shared_haplo : incompatibilty between haplotype length and tested position.")
      end if

      tab_shared_haplo=0
!
!  recherche du marqueur a gauche de dx
      do lkleft=1,nmk(c)-1
        if(dx <= posi(c,lkleft)) exit
      end do
!
      do ip=1,np
        do jm=nmp(ip)+1,nmp(ip+1)
          do kd=ndm(jm)+1,ndm(jm+1)
            icount_max=0
            shared_haplo(kd)=0
!
! icount est la longueur du segment de l'haplotype maternel ibs au chromosome iall du pere jp
            do jp=1,np
              do iall=1,2
                icount=0
                do lk=lkleft,1,-1
                  if(tab_IBS(kd,lk,2) /= genotyp(c,lk,correp(jp),iall)) exit
                  if(genotyp(c,lk,correp(jp),iall)/= nmanque) icount=icount+1
                end do
                do lk=lkleft+1,nmk(c)
                  if(tab_IBS(kd,lk,2) /= genotyp(c,lk,correp(jp),iall)) exit
                  if(genotyp(c,lk,correp(jp),iall)/= nmanque) icount=icount+1
                end do

                if(icount > icount_max .and. icount >= LONG_MIN_IBS) then
                  icount_max=icount
                  shared_haplo(kd)=iall+2*(jp-1)
                end if
              end do ! iall
            end do ! jp

            if( shared_haplo(kd) /= 0) then
              iall=2-modulo(shared_haplo(kd),2)
              jp=(shared_haplo(kd)+modulo(shared_haplo(kd),2))/2
              tab_shared_haplo(jp,iall,1+icount_max)=tab_shared_haplo(jp,iall,1+icount_max)+1 !nombre d'haplotyped e longueur count_max
              tab_shared_haplo(jp,iall,1)=tab_shared_haplo(jp,iall,1)+1 ! nombre d haplotype partage
            end if
          end do !kd
        end do !jm
      end do !ip
!
      end subroutine liste_shared_haplo
!!***

!!****f* m_qtlmap_haplotype_ldla/bilan_shared_haplo
!! NAME
!!    bilan_shared_haplo
!! DESCRIPTION
!!
!! NOTES
!!
!! bilan_shared_haplo  informe sur le nombre et les carateristiques des haplotypes paternels
!! retrouves chez les meres
!! Utilise nmk,np,posi,correp,nm                         de m_qtlmap_data
!! Utilise genotyp                                       de m_qtlmap_haplotype_data
!! SOURCE
       subroutine bilan_shared_haplo(c,dx,n,ip)

!
      integer       , intent(in) :: c,ip
      real(kind=dp) , intent(in) :: dx
      integer, intent (in) :: n
      integer itab,jtab,ktab,stat,imax1,imax2
      integer  , dimension (:,:)  , allocatable   :: tab
! allocation
!
      allocate (tab(2,maxval(nmk)),STAT=stat)
      call check_allocate(stat,'tab [m_qtlmap_haplotype_LD]')

      if(n ==1 .and. ip ==1)then
        write(nficout,20) ip
   20   format(' Dam carried Sire ',i6,' haplotypes length distribution (grouped by 20 SNP)')
        write(nficout,30)
   30   format(' position *  sire  * 1st haplotype / 2nd haplotype'//72('*')//&
               '          *        *  10  20  30  40  50  60  70  80  90 100 110 120',&
                                     ' 130 140 150 160 170 180 190 200.....'// 72('*'))
       end if

      tab=0
      jtab=0
      ktab=1
      do itab =1,nmk(c)
        jtab=jtab+1
        tab(1,ktab)=tab(1,ktab)+ tab_shared_haplo(ip,1,1+itab)
        tab(2,ktab)=tab(2,ktab)+ tab_shared_haplo(ip,2,1+itab)
        if(jtab == 10) then
          jtab=0
          ktab=ktab+1
        end if
      end do
      do imax1 =ktab,1,-1
        if(tab(1,imax1) /=0) exit
      end do
      do imax2 =ktab,1,-1
        if(tab(2,imax2) /=0) exit
      end do

      write(nficout,31)  dx,ip,(tab(1,itab),itab=1,imax1)
   31 format(3x,f5.3,'  *',i8,'*',30(1x,i3))
     write(nficout,32)      (tab(2,itab),itab=1,imax2)
   32 format(3x,5x,11x,'*',30(1x,i3))
      deallocate (tab)

      end subroutine bilan_shared_haplo
!!***

      end module m_qtlmap_haplotype_ldla
